# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'AclAclRule',
    'ClusterAutoUpgrade',
    'ClusterAutoscalerConfig',
    'ClusterKubeconfig',
    'ClusterOpenIdConnectConfig',
    'PoolNode',
    'PoolNodePrivateIp',
    'PoolUpgradePolicy',
    'GetClusterAutoUpgradeResult',
    'GetClusterAutoscalerConfigResult',
    'GetClusterKubeconfigResult',
    'GetClusterOpenIdConnectConfigResult',
    'GetPoolNodeResult',
    'GetPoolNodePrivateIpResult',
    'GetPoolUpgradePolicyResult',
]

@pulumi.output_type
class AclAclRule(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "scalewayRanges":
            suggest = "scaleway_ranges"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in AclAclRule. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        AclAclRule.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        AclAclRule.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 description: Optional[_builtins.str] = None,
                 id: Optional[_builtins.str] = None,
                 ip: Optional[_builtins.str] = None,
                 scaleway_ranges: Optional[_builtins.bool] = None):
        """
        :param _builtins.str description: A text describing this rule.
        :param _builtins.str id: The ID of the ACL resource. It is the same as the ID of the cluster.
        :param _builtins.str ip: The IP range to whitelist in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)
               
               > **Important:** If the `ip` field is set, `scaleway_ranges` cannot be set to true in the same rule.
        :param _builtins.bool scaleway_ranges: Allow access to cluster from all Scaleway ranges as defined in [Scaleway Network Information - IP ranges used by Scaleway](https://www.scaleway.com/en/docs/console/account/reference-content/scaleway-network-information/#ip-ranges-used-by-scaleway).
               Only one rule with this field set to true can be added.
               
               > **Important:** If the `scaleway_ranges` field is set to true, the `ip` field cannot be set on the same rule.
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if id is not None:
            pulumi.set(__self__, "id", id)
        if ip is not None:
            pulumi.set(__self__, "ip", ip)
        if scaleway_ranges is not None:
            pulumi.set(__self__, "scaleway_ranges", scaleway_ranges)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[_builtins.str]:
        """
        A text describing this rule.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[_builtins.str]:
        """
        The ID of the ACL resource. It is the same as the ID of the cluster.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def ip(self) -> Optional[_builtins.str]:
        """
        The IP range to whitelist in [CIDR notation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation)

        > **Important:** If the `ip` field is set, `scaleway_ranges` cannot be set to true in the same rule.
        """
        return pulumi.get(self, "ip")

    @_builtins.property
    @pulumi.getter(name="scalewayRanges")
    def scaleway_ranges(self) -> Optional[_builtins.bool]:
        """
        Allow access to cluster from all Scaleway ranges as defined in [Scaleway Network Information - IP ranges used by Scaleway](https://www.scaleway.com/en/docs/console/account/reference-content/scaleway-network-information/#ip-ranges-used-by-scaleway).
        Only one rule with this field set to true can be added.

        > **Important:** If the `scaleway_ranges` field is set to true, the `ip` field cannot be set on the same rule.
        """
        return pulumi.get(self, "scaleway_ranges")


@pulumi.output_type
class ClusterAutoUpgrade(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "maintenanceWindowDay":
            suggest = "maintenance_window_day"
        elif key == "maintenanceWindowStartHour":
            suggest = "maintenance_window_start_hour"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ClusterAutoUpgrade. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ClusterAutoUpgrade.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ClusterAutoUpgrade.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 enable: _builtins.bool,
                 maintenance_window_day: _builtins.str,
                 maintenance_window_start_hour: _builtins.int):
        """
        :param _builtins.bool enable: Set to `true` to enable Kubernetes patch version auto upgrades.
               > **Important:** When enabling auto upgrades, the `version` field take a minor version like x.y (ie 1.18).
        :param _builtins.str maintenance_window_day: The day of the auto upgrade maintenance window (`monday` to `sunday`, or `any`).
        :param _builtins.int maintenance_window_start_hour: The start hour (UTC) of the 2-hour auto upgrade maintenance window (0 to 23).
        """
        pulumi.set(__self__, "enable", enable)
        pulumi.set(__self__, "maintenance_window_day", maintenance_window_day)
        pulumi.set(__self__, "maintenance_window_start_hour", maintenance_window_start_hour)

    @_builtins.property
    @pulumi.getter
    def enable(self) -> _builtins.bool:
        """
        Set to `true` to enable Kubernetes patch version auto upgrades.
        > **Important:** When enabling auto upgrades, the `version` field take a minor version like x.y (ie 1.18).
        """
        return pulumi.get(self, "enable")

    @_builtins.property
    @pulumi.getter(name="maintenanceWindowDay")
    def maintenance_window_day(self) -> _builtins.str:
        """
        The day of the auto upgrade maintenance window (`monday` to `sunday`, or `any`).
        """
        return pulumi.get(self, "maintenance_window_day")

    @_builtins.property
    @pulumi.getter(name="maintenanceWindowStartHour")
    def maintenance_window_start_hour(self) -> _builtins.int:
        """
        The start hour (UTC) of the 2-hour auto upgrade maintenance window (0 to 23).
        """
        return pulumi.get(self, "maintenance_window_start_hour")


@pulumi.output_type
class ClusterAutoscalerConfig(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "balanceSimilarNodeGroups":
            suggest = "balance_similar_node_groups"
        elif key == "disableScaleDown":
            suggest = "disable_scale_down"
        elif key == "expendablePodsPriorityCutoff":
            suggest = "expendable_pods_priority_cutoff"
        elif key == "ignoreDaemonsetsUtilization":
            suggest = "ignore_daemonsets_utilization"
        elif key == "maxGracefulTerminationSec":
            suggest = "max_graceful_termination_sec"
        elif key == "scaleDownDelayAfterAdd":
            suggest = "scale_down_delay_after_add"
        elif key == "scaleDownUnneededTime":
            suggest = "scale_down_unneeded_time"
        elif key == "scaleDownUtilizationThreshold":
            suggest = "scale_down_utilization_threshold"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ClusterAutoscalerConfig. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ClusterAutoscalerConfig.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ClusterAutoscalerConfig.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 balance_similar_node_groups: Optional[_builtins.bool] = None,
                 disable_scale_down: Optional[_builtins.bool] = None,
                 estimator: Optional[_builtins.str] = None,
                 expander: Optional[_builtins.str] = None,
                 expendable_pods_priority_cutoff: Optional[_builtins.int] = None,
                 ignore_daemonsets_utilization: Optional[_builtins.bool] = None,
                 max_graceful_termination_sec: Optional[_builtins.int] = None,
                 scale_down_delay_after_add: Optional[_builtins.str] = None,
                 scale_down_unneeded_time: Optional[_builtins.str] = None,
                 scale_down_utilization_threshold: Optional[_builtins.float] = None):
        """
        :param _builtins.bool balance_similar_node_groups: Detect similar node groups and balance the number of nodes between them.
        :param _builtins.bool disable_scale_down: Disables the scale down feature of the autoscaler.
        :param _builtins.str estimator: Type of resource estimator to be used in scale up.
        :param _builtins.str expander: Type of node group expander to be used in scale up.
        :param _builtins.int expendable_pods_priority_cutoff: Pods with priority below cutoff will be expendable. They can be killed without any consideration during scale down and they don't cause scale up. Pods with null priority (PodPriority disabled) are non expendable.
        :param _builtins.bool ignore_daemonsets_utilization: Ignore DaemonSet pods when calculating resource utilization for scaling down.
        :param _builtins.int max_graceful_termination_sec: Maximum number of seconds the cluster autoscaler waits for pod termination when trying to scale down a node
        :param _builtins.str scale_down_delay_after_add: How long after scale up that scale down evaluation resumes.
        :param _builtins.str scale_down_unneeded_time: How long a node should be unneeded before it is eligible for scale down.
        :param _builtins.float scale_down_utilization_threshold: Node utilization level, defined as sum of requested resources divided by capacity, below which a node can be considered for scale down
        """
        if balance_similar_node_groups is not None:
            pulumi.set(__self__, "balance_similar_node_groups", balance_similar_node_groups)
        if disable_scale_down is not None:
            pulumi.set(__self__, "disable_scale_down", disable_scale_down)
        if estimator is not None:
            pulumi.set(__self__, "estimator", estimator)
        if expander is not None:
            pulumi.set(__self__, "expander", expander)
        if expendable_pods_priority_cutoff is not None:
            pulumi.set(__self__, "expendable_pods_priority_cutoff", expendable_pods_priority_cutoff)
        if ignore_daemonsets_utilization is not None:
            pulumi.set(__self__, "ignore_daemonsets_utilization", ignore_daemonsets_utilization)
        if max_graceful_termination_sec is not None:
            pulumi.set(__self__, "max_graceful_termination_sec", max_graceful_termination_sec)
        if scale_down_delay_after_add is not None:
            pulumi.set(__self__, "scale_down_delay_after_add", scale_down_delay_after_add)
        if scale_down_unneeded_time is not None:
            pulumi.set(__self__, "scale_down_unneeded_time", scale_down_unneeded_time)
        if scale_down_utilization_threshold is not None:
            pulumi.set(__self__, "scale_down_utilization_threshold", scale_down_utilization_threshold)

    @_builtins.property
    @pulumi.getter(name="balanceSimilarNodeGroups")
    def balance_similar_node_groups(self) -> Optional[_builtins.bool]:
        """
        Detect similar node groups and balance the number of nodes between them.
        """
        return pulumi.get(self, "balance_similar_node_groups")

    @_builtins.property
    @pulumi.getter(name="disableScaleDown")
    def disable_scale_down(self) -> Optional[_builtins.bool]:
        """
        Disables the scale down feature of the autoscaler.
        """
        return pulumi.get(self, "disable_scale_down")

    @_builtins.property
    @pulumi.getter
    def estimator(self) -> Optional[_builtins.str]:
        """
        Type of resource estimator to be used in scale up.
        """
        return pulumi.get(self, "estimator")

    @_builtins.property
    @pulumi.getter
    def expander(self) -> Optional[_builtins.str]:
        """
        Type of node group expander to be used in scale up.
        """
        return pulumi.get(self, "expander")

    @_builtins.property
    @pulumi.getter(name="expendablePodsPriorityCutoff")
    def expendable_pods_priority_cutoff(self) -> Optional[_builtins.int]:
        """
        Pods with priority below cutoff will be expendable. They can be killed without any consideration during scale down and they don't cause scale up. Pods with null priority (PodPriority disabled) are non expendable.
        """
        return pulumi.get(self, "expendable_pods_priority_cutoff")

    @_builtins.property
    @pulumi.getter(name="ignoreDaemonsetsUtilization")
    def ignore_daemonsets_utilization(self) -> Optional[_builtins.bool]:
        """
        Ignore DaemonSet pods when calculating resource utilization for scaling down.
        """
        return pulumi.get(self, "ignore_daemonsets_utilization")

    @_builtins.property
    @pulumi.getter(name="maxGracefulTerminationSec")
    def max_graceful_termination_sec(self) -> Optional[_builtins.int]:
        """
        Maximum number of seconds the cluster autoscaler waits for pod termination when trying to scale down a node
        """
        return pulumi.get(self, "max_graceful_termination_sec")

    @_builtins.property
    @pulumi.getter(name="scaleDownDelayAfterAdd")
    def scale_down_delay_after_add(self) -> Optional[_builtins.str]:
        """
        How long after scale up that scale down evaluation resumes.
        """
        return pulumi.get(self, "scale_down_delay_after_add")

    @_builtins.property
    @pulumi.getter(name="scaleDownUnneededTime")
    def scale_down_unneeded_time(self) -> Optional[_builtins.str]:
        """
        How long a node should be unneeded before it is eligible for scale down.
        """
        return pulumi.get(self, "scale_down_unneeded_time")

    @_builtins.property
    @pulumi.getter(name="scaleDownUtilizationThreshold")
    def scale_down_utilization_threshold(self) -> Optional[_builtins.float]:
        """
        Node utilization level, defined as sum of requested resources divided by capacity, below which a node can be considered for scale down
        """
        return pulumi.get(self, "scale_down_utilization_threshold")


@pulumi.output_type
class ClusterKubeconfig(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "clusterCaCertificate":
            suggest = "cluster_ca_certificate"
        elif key == "configFile":
            suggest = "config_file"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ClusterKubeconfig. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ClusterKubeconfig.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ClusterKubeconfig.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 cluster_ca_certificate: Optional[_builtins.str] = None,
                 config_file: Optional[_builtins.str] = None,
                 host: Optional[_builtins.str] = None,
                 token: Optional[_builtins.str] = None):
        """
        :param _builtins.str cluster_ca_certificate: The CA certificate of the Kubernetes API server.
        :param _builtins.str config_file: The raw kubeconfig file.
        :param _builtins.str host: The URL of the Kubernetes API server.
        :param _builtins.str token: The token to connect to the Kubernetes API server.
        """
        if cluster_ca_certificate is not None:
            pulumi.set(__self__, "cluster_ca_certificate", cluster_ca_certificate)
        if config_file is not None:
            pulumi.set(__self__, "config_file", config_file)
        if host is not None:
            pulumi.set(__self__, "host", host)
        if token is not None:
            pulumi.set(__self__, "token", token)

    @_builtins.property
    @pulumi.getter(name="clusterCaCertificate")
    def cluster_ca_certificate(self) -> Optional[_builtins.str]:
        """
        The CA certificate of the Kubernetes API server.
        """
        return pulumi.get(self, "cluster_ca_certificate")

    @_builtins.property
    @pulumi.getter(name="configFile")
    def config_file(self) -> Optional[_builtins.str]:
        """
        The raw kubeconfig file.
        """
        return pulumi.get(self, "config_file")

    @_builtins.property
    @pulumi.getter
    def host(self) -> Optional[_builtins.str]:
        """
        The URL of the Kubernetes API server.
        """
        return pulumi.get(self, "host")

    @_builtins.property
    @pulumi.getter
    def token(self) -> Optional[_builtins.str]:
        """
        The token to connect to the Kubernetes API server.
        """
        return pulumi.get(self, "token")


@pulumi.output_type
class ClusterOpenIdConnectConfig(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "clientId":
            suggest = "client_id"
        elif key == "issuerUrl":
            suggest = "issuer_url"
        elif key == "groupsClaims":
            suggest = "groups_claims"
        elif key == "groupsPrefix":
            suggest = "groups_prefix"
        elif key == "requiredClaims":
            suggest = "required_claims"
        elif key == "usernameClaim":
            suggest = "username_claim"
        elif key == "usernamePrefix":
            suggest = "username_prefix"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ClusterOpenIdConnectConfig. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ClusterOpenIdConnectConfig.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ClusterOpenIdConnectConfig.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 client_id: _builtins.str,
                 issuer_url: _builtins.str,
                 groups_claims: Optional[Sequence[_builtins.str]] = None,
                 groups_prefix: Optional[_builtins.str] = None,
                 required_claims: Optional[Sequence[_builtins.str]] = None,
                 username_claim: Optional[_builtins.str] = None,
                 username_prefix: Optional[_builtins.str] = None):
        """
        :param _builtins.str client_id: A client id that all tokens must be issued for
        :param _builtins.str issuer_url: URL of the provider which allows the API server to discover public signing keys
        :param Sequence[_builtins.str] groups_claims: JWT claim to use as the user's group
        :param _builtins.str groups_prefix: Prefix prepended to group claims
        :param Sequence[_builtins.str] required_claims: Multiple key=value pairs that describes a required claim in the ID Token
        :param _builtins.str username_claim: JWT claim to use as the user name
        :param _builtins.str username_prefix: Prefix prepended to username
        """
        pulumi.set(__self__, "client_id", client_id)
        pulumi.set(__self__, "issuer_url", issuer_url)
        if groups_claims is not None:
            pulumi.set(__self__, "groups_claims", groups_claims)
        if groups_prefix is not None:
            pulumi.set(__self__, "groups_prefix", groups_prefix)
        if required_claims is not None:
            pulumi.set(__self__, "required_claims", required_claims)
        if username_claim is not None:
            pulumi.set(__self__, "username_claim", username_claim)
        if username_prefix is not None:
            pulumi.set(__self__, "username_prefix", username_prefix)

    @_builtins.property
    @pulumi.getter(name="clientId")
    def client_id(self) -> _builtins.str:
        """
        A client id that all tokens must be issued for
        """
        return pulumi.get(self, "client_id")

    @_builtins.property
    @pulumi.getter(name="issuerUrl")
    def issuer_url(self) -> _builtins.str:
        """
        URL of the provider which allows the API server to discover public signing keys
        """
        return pulumi.get(self, "issuer_url")

    @_builtins.property
    @pulumi.getter(name="groupsClaims")
    def groups_claims(self) -> Optional[Sequence[_builtins.str]]:
        """
        JWT claim to use as the user's group
        """
        return pulumi.get(self, "groups_claims")

    @_builtins.property
    @pulumi.getter(name="groupsPrefix")
    def groups_prefix(self) -> Optional[_builtins.str]:
        """
        Prefix prepended to group claims
        """
        return pulumi.get(self, "groups_prefix")

    @_builtins.property
    @pulumi.getter(name="requiredClaims")
    def required_claims(self) -> Optional[Sequence[_builtins.str]]:
        """
        Multiple key=value pairs that describes a required claim in the ID Token
        """
        return pulumi.get(self, "required_claims")

    @_builtins.property
    @pulumi.getter(name="usernameClaim")
    def username_claim(self) -> Optional[_builtins.str]:
        """
        JWT claim to use as the user name
        """
        return pulumi.get(self, "username_claim")

    @_builtins.property
    @pulumi.getter(name="usernamePrefix")
    def username_prefix(self) -> Optional[_builtins.str]:
        """
        Prefix prepended to username
        """
        return pulumi.get(self, "username_prefix")


@pulumi.output_type
class PoolNode(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "privateIps":
            suggest = "private_ips"
        elif key == "publicIp":
            suggest = "public_ip"
        elif key == "publicIpV6":
            suggest = "public_ip_v6"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in PoolNode. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        PoolNode.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        PoolNode.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 id: Optional[_builtins.str] = None,
                 name: Optional[_builtins.str] = None,
                 private_ips: Optional[Sequence['outputs.PoolNodePrivateIp']] = None,
                 public_ip: Optional[_builtins.str] = None,
                 public_ip_v6: Optional[_builtins.str] = None,
                 status: Optional[_builtins.str] = None):
        """
        :param _builtins.str id: The ID of the IP address resource.
        :param _builtins.str name: The name for the pool.
               
               > **Important:** Updates to this field will recreate a new resource.
        :param Sequence['PoolNodePrivateIpArgs'] private_ips: The list of private IPv4 and IPv6 addresses associated with the node.
        :param _builtins.str public_ip: The public IPv4. (Deprecated, Please use the official Kubernetes provider and the kubernetes_nodes data source)
        :param _builtins.str public_ip_v6: The public IPv6. (Deprecated, Please use the official Kubernetes provider and the kubernetes_nodes data source)
        :param _builtins.str status: The status of the node.
        """
        if id is not None:
            pulumi.set(__self__, "id", id)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if private_ips is not None:
            pulumi.set(__self__, "private_ips", private_ips)
        if public_ip is not None:
            pulumi.set(__self__, "public_ip", public_ip)
        if public_ip_v6 is not None:
            pulumi.set(__self__, "public_ip_v6", public_ip_v6)
        if status is not None:
            pulumi.set(__self__, "status", status)

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[_builtins.str]:
        """
        The ID of the IP address resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        """
        The name for the pool.

        > **Important:** Updates to this field will recreate a new resource.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="privateIps")
    def private_ips(self) -> Optional[Sequence['outputs.PoolNodePrivateIp']]:
        """
        The list of private IPv4 and IPv6 addresses associated with the node.
        """
        return pulumi.get(self, "private_ips")

    @_builtins.property
    @pulumi.getter(name="publicIp")
    @_utilities.deprecated("""Please use the official Kubernetes provider and the kubernetes_nodes data source""")
    def public_ip(self) -> Optional[_builtins.str]:
        """
        The public IPv4. (Deprecated, Please use the official Kubernetes provider and the kubernetes_nodes data source)
        """
        return pulumi.get(self, "public_ip")

    @_builtins.property
    @pulumi.getter(name="publicIpV6")
    @_utilities.deprecated("""Please use the official Kubernetes provider and the kubernetes_nodes data source""")
    def public_ip_v6(self) -> Optional[_builtins.str]:
        """
        The public IPv6. (Deprecated, Please use the official Kubernetes provider and the kubernetes_nodes data source)
        """
        return pulumi.get(self, "public_ip_v6")

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[_builtins.str]:
        """
        The status of the node.
        """
        return pulumi.get(self, "status")


@pulumi.output_type
class PoolNodePrivateIp(dict):
    def __init__(__self__, *,
                 address: Optional[_builtins.str] = None,
                 id: Optional[_builtins.str] = None):
        """
        :param _builtins.str address: The private IP address.
        :param _builtins.str id: The ID of the IP address resource.
        """
        if address is not None:
            pulumi.set(__self__, "address", address)
        if id is not None:
            pulumi.set(__self__, "id", id)

    @_builtins.property
    @pulumi.getter
    def address(self) -> Optional[_builtins.str]:
        """
        The private IP address.
        """
        return pulumi.get(self, "address")

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[_builtins.str]:
        """
        The ID of the IP address resource.
        """
        return pulumi.get(self, "id")


@pulumi.output_type
class PoolUpgradePolicy(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "maxSurge":
            suggest = "max_surge"
        elif key == "maxUnavailable":
            suggest = "max_unavailable"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in PoolUpgradePolicy. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        PoolUpgradePolicy.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        PoolUpgradePolicy.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 max_surge: Optional[_builtins.int] = None,
                 max_unavailable: Optional[_builtins.int] = None):
        """
        :param _builtins.int max_surge: The maximum number of nodes to be created during the upgrade
        :param _builtins.int max_unavailable: The maximum number of nodes that can be not ready at the same time
        """
        if max_surge is not None:
            pulumi.set(__self__, "max_surge", max_surge)
        if max_unavailable is not None:
            pulumi.set(__self__, "max_unavailable", max_unavailable)

    @_builtins.property
    @pulumi.getter(name="maxSurge")
    def max_surge(self) -> Optional[_builtins.int]:
        """
        The maximum number of nodes to be created during the upgrade
        """
        return pulumi.get(self, "max_surge")

    @_builtins.property
    @pulumi.getter(name="maxUnavailable")
    def max_unavailable(self) -> Optional[_builtins.int]:
        """
        The maximum number of nodes that can be not ready at the same time
        """
        return pulumi.get(self, "max_unavailable")


@pulumi.output_type
class GetClusterAutoUpgradeResult(dict):
    def __init__(__self__, *,
                 enable: _builtins.bool,
                 maintenance_window_day: _builtins.str,
                 maintenance_window_start_hour: _builtins.int):
        """
        :param _builtins.bool enable: True if Kubernetes patch version auto upgrades is enabled.
        :param _builtins.str maintenance_window_day: The day of the auto upgrade maintenance window (`monday` to `sunday`, or `any`).
        :param _builtins.int maintenance_window_start_hour: The start hour (UTC) of the 2-hour auto upgrade maintenance window (0 to 23).
        """
        pulumi.set(__self__, "enable", enable)
        pulumi.set(__self__, "maintenance_window_day", maintenance_window_day)
        pulumi.set(__self__, "maintenance_window_start_hour", maintenance_window_start_hour)

    @_builtins.property
    @pulumi.getter
    def enable(self) -> _builtins.bool:
        """
        True if Kubernetes patch version auto upgrades is enabled.
        """
        return pulumi.get(self, "enable")

    @_builtins.property
    @pulumi.getter(name="maintenanceWindowDay")
    def maintenance_window_day(self) -> _builtins.str:
        """
        The day of the auto upgrade maintenance window (`monday` to `sunday`, or `any`).
        """
        return pulumi.get(self, "maintenance_window_day")

    @_builtins.property
    @pulumi.getter(name="maintenanceWindowStartHour")
    def maintenance_window_start_hour(self) -> _builtins.int:
        """
        The start hour (UTC) of the 2-hour auto upgrade maintenance window (0 to 23).
        """
        return pulumi.get(self, "maintenance_window_start_hour")


@pulumi.output_type
class GetClusterAutoscalerConfigResult(dict):
    def __init__(__self__, *,
                 balance_similar_node_groups: _builtins.bool,
                 disable_scale_down: _builtins.bool,
                 estimator: _builtins.str,
                 expander: _builtins.str,
                 expendable_pods_priority_cutoff: _builtins.int,
                 ignore_daemonsets_utilization: _builtins.bool,
                 max_graceful_termination_sec: _builtins.int,
                 scale_down_delay_after_add: _builtins.str,
                 scale_down_unneeded_time: _builtins.str,
                 scale_down_utilization_threshold: _builtins.float):
        """
        :param _builtins.bool balance_similar_node_groups: True if detecting similar node groups and balance the number of nodes between them is enabled.
        :param _builtins.bool disable_scale_down: True if the scale down feature of the autoscaler is disabled.
        :param _builtins.str estimator: The type of resource estimator used in scale up.
        :param _builtins.str expander: The type of node group expander be used in scale up.
        :param _builtins.int expendable_pods_priority_cutoff: Pods with priority below cutoff will be expendable. They can be killed without any consideration during scale down and they don't cause scale up. Pods with null priority (PodPriority disabled) are non expendable.
        :param _builtins.bool ignore_daemonsets_utilization: True if ignoring DaemonSet pods when calculating resource utilization for scaling down is enabled.
        :param _builtins.int max_graceful_termination_sec: Maximum number of seconds the cluster autoscaler waits for pod termination when trying to scale down a node
        :param _builtins.str scale_down_delay_after_add: The duration after scale up that scale down evaluation resumes.
        :param _builtins.str scale_down_unneeded_time: The duration a node should be unneeded before it is eligible for scale down.
        :param _builtins.float scale_down_utilization_threshold: Node utilization level, defined as sum of requested resources divided by capacity, below which a node can be considered for scale down
        """
        pulumi.set(__self__, "balance_similar_node_groups", balance_similar_node_groups)
        pulumi.set(__self__, "disable_scale_down", disable_scale_down)
        pulumi.set(__self__, "estimator", estimator)
        pulumi.set(__self__, "expander", expander)
        pulumi.set(__self__, "expendable_pods_priority_cutoff", expendable_pods_priority_cutoff)
        pulumi.set(__self__, "ignore_daemonsets_utilization", ignore_daemonsets_utilization)
        pulumi.set(__self__, "max_graceful_termination_sec", max_graceful_termination_sec)
        pulumi.set(__self__, "scale_down_delay_after_add", scale_down_delay_after_add)
        pulumi.set(__self__, "scale_down_unneeded_time", scale_down_unneeded_time)
        pulumi.set(__self__, "scale_down_utilization_threshold", scale_down_utilization_threshold)

    @_builtins.property
    @pulumi.getter(name="balanceSimilarNodeGroups")
    def balance_similar_node_groups(self) -> _builtins.bool:
        """
        True if detecting similar node groups and balance the number of nodes between them is enabled.
        """
        return pulumi.get(self, "balance_similar_node_groups")

    @_builtins.property
    @pulumi.getter(name="disableScaleDown")
    def disable_scale_down(self) -> _builtins.bool:
        """
        True if the scale down feature of the autoscaler is disabled.
        """
        return pulumi.get(self, "disable_scale_down")

    @_builtins.property
    @pulumi.getter
    def estimator(self) -> _builtins.str:
        """
        The type of resource estimator used in scale up.
        """
        return pulumi.get(self, "estimator")

    @_builtins.property
    @pulumi.getter
    def expander(self) -> _builtins.str:
        """
        The type of node group expander be used in scale up.
        """
        return pulumi.get(self, "expander")

    @_builtins.property
    @pulumi.getter(name="expendablePodsPriorityCutoff")
    def expendable_pods_priority_cutoff(self) -> _builtins.int:
        """
        Pods with priority below cutoff will be expendable. They can be killed without any consideration during scale down and they don't cause scale up. Pods with null priority (PodPriority disabled) are non expendable.
        """
        return pulumi.get(self, "expendable_pods_priority_cutoff")

    @_builtins.property
    @pulumi.getter(name="ignoreDaemonsetsUtilization")
    def ignore_daemonsets_utilization(self) -> _builtins.bool:
        """
        True if ignoring DaemonSet pods when calculating resource utilization for scaling down is enabled.
        """
        return pulumi.get(self, "ignore_daemonsets_utilization")

    @_builtins.property
    @pulumi.getter(name="maxGracefulTerminationSec")
    def max_graceful_termination_sec(self) -> _builtins.int:
        """
        Maximum number of seconds the cluster autoscaler waits for pod termination when trying to scale down a node
        """
        return pulumi.get(self, "max_graceful_termination_sec")

    @_builtins.property
    @pulumi.getter(name="scaleDownDelayAfterAdd")
    def scale_down_delay_after_add(self) -> _builtins.str:
        """
        The duration after scale up that scale down evaluation resumes.
        """
        return pulumi.get(self, "scale_down_delay_after_add")

    @_builtins.property
    @pulumi.getter(name="scaleDownUnneededTime")
    def scale_down_unneeded_time(self) -> _builtins.str:
        """
        The duration a node should be unneeded before it is eligible for scale down.
        """
        return pulumi.get(self, "scale_down_unneeded_time")

    @_builtins.property
    @pulumi.getter(name="scaleDownUtilizationThreshold")
    def scale_down_utilization_threshold(self) -> _builtins.float:
        """
        Node utilization level, defined as sum of requested resources divided by capacity, below which a node can be considered for scale down
        """
        return pulumi.get(self, "scale_down_utilization_threshold")


@pulumi.output_type
class GetClusterKubeconfigResult(dict):
    def __init__(__self__, *,
                 cluster_ca_certificate: _builtins.str,
                 config_file: _builtins.str,
                 host: _builtins.str,
                 token: _builtins.str):
        """
        :param _builtins.str cluster_ca_certificate: The CA certificate of the Kubernetes API server.
        :param _builtins.str config_file: The raw kubeconfig file.
        :param _builtins.str host: The URL of the Kubernetes API server.
        :param _builtins.str token: The token to connect to the Kubernetes API server.
        """
        pulumi.set(__self__, "cluster_ca_certificate", cluster_ca_certificate)
        pulumi.set(__self__, "config_file", config_file)
        pulumi.set(__self__, "host", host)
        pulumi.set(__self__, "token", token)

    @_builtins.property
    @pulumi.getter(name="clusterCaCertificate")
    def cluster_ca_certificate(self) -> _builtins.str:
        """
        The CA certificate of the Kubernetes API server.
        """
        return pulumi.get(self, "cluster_ca_certificate")

    @_builtins.property
    @pulumi.getter(name="configFile")
    def config_file(self) -> _builtins.str:
        """
        The raw kubeconfig file.
        """
        return pulumi.get(self, "config_file")

    @_builtins.property
    @pulumi.getter
    def host(self) -> _builtins.str:
        """
        The URL of the Kubernetes API server.
        """
        return pulumi.get(self, "host")

    @_builtins.property
    @pulumi.getter
    def token(self) -> _builtins.str:
        """
        The token to connect to the Kubernetes API server.
        """
        return pulumi.get(self, "token")


@pulumi.output_type
class GetClusterOpenIdConnectConfigResult(dict):
    def __init__(__self__, *,
                 client_id: _builtins.str,
                 groups_claims: Sequence[_builtins.str],
                 groups_prefix: _builtins.str,
                 issuer_url: _builtins.str,
                 required_claims: Sequence[_builtins.str],
                 username_claim: _builtins.str,
                 username_prefix: _builtins.str):
        """
        :param _builtins.str client_id: A client id that all tokens must be issued for
        :param Sequence[_builtins.str] groups_claims: JWT claim to use as the user's group
        :param _builtins.str groups_prefix: Prefix prepended to group claims
        :param _builtins.str issuer_url: URL of the provider which allows the API server to discover public signing keys
        :param Sequence[_builtins.str] required_claims: Multiple key=value pairs that describes a required claim in the ID Token
        :param _builtins.str username_claim: JWT claim to use as the user name
        :param _builtins.str username_prefix: Prefix prepended to username
        """
        pulumi.set(__self__, "client_id", client_id)
        pulumi.set(__self__, "groups_claims", groups_claims)
        pulumi.set(__self__, "groups_prefix", groups_prefix)
        pulumi.set(__self__, "issuer_url", issuer_url)
        pulumi.set(__self__, "required_claims", required_claims)
        pulumi.set(__self__, "username_claim", username_claim)
        pulumi.set(__self__, "username_prefix", username_prefix)

    @_builtins.property
    @pulumi.getter(name="clientId")
    def client_id(self) -> _builtins.str:
        """
        A client id that all tokens must be issued for
        """
        return pulumi.get(self, "client_id")

    @_builtins.property
    @pulumi.getter(name="groupsClaims")
    def groups_claims(self) -> Sequence[_builtins.str]:
        """
        JWT claim to use as the user's group
        """
        return pulumi.get(self, "groups_claims")

    @_builtins.property
    @pulumi.getter(name="groupsPrefix")
    def groups_prefix(self) -> _builtins.str:
        """
        Prefix prepended to group claims
        """
        return pulumi.get(self, "groups_prefix")

    @_builtins.property
    @pulumi.getter(name="issuerUrl")
    def issuer_url(self) -> _builtins.str:
        """
        URL of the provider which allows the API server to discover public signing keys
        """
        return pulumi.get(self, "issuer_url")

    @_builtins.property
    @pulumi.getter(name="requiredClaims")
    def required_claims(self) -> Sequence[_builtins.str]:
        """
        Multiple key=value pairs that describes a required claim in the ID Token
        """
        return pulumi.get(self, "required_claims")

    @_builtins.property
    @pulumi.getter(name="usernameClaim")
    def username_claim(self) -> _builtins.str:
        """
        JWT claim to use as the user name
        """
        return pulumi.get(self, "username_claim")

    @_builtins.property
    @pulumi.getter(name="usernamePrefix")
    def username_prefix(self) -> _builtins.str:
        """
        Prefix prepended to username
        """
        return pulumi.get(self, "username_prefix")


@pulumi.output_type
class GetPoolNodeResult(dict):
    def __init__(__self__, *,
                 id: _builtins.str,
                 name: _builtins.str,
                 private_ips: Sequence['outputs.GetPoolNodePrivateIpResult'],
                 public_ip: _builtins.str,
                 public_ip_v6: _builtins.str,
                 status: _builtins.str):
        """
        :param _builtins.str id: The ID of the pool.
        :param _builtins.str name: The pool name. Only one of `name` and `pool_id` should be specified. `cluster_id` should be specified with `name`.
        :param Sequence['GetPoolNodePrivateIpArgs'] private_ips: List of private IPv4 and IPv6 addresses associated with the node
        :param _builtins.str public_ip: The public IPv4.
        :param _builtins.str public_ip_v6: The public IPv6.
        :param _builtins.str status: The status of the node.
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "private_ips", private_ips)
        pulumi.set(__self__, "public_ip", public_ip)
        pulumi.set(__self__, "public_ip_v6", public_ip_v6)
        pulumi.set(__self__, "status", status)

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The ID of the pool.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        The pool name. Only one of `name` and `pool_id` should be specified. `cluster_id` should be specified with `name`.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="privateIps")
    def private_ips(self) -> Sequence['outputs.GetPoolNodePrivateIpResult']:
        """
        List of private IPv4 and IPv6 addresses associated with the node
        """
        return pulumi.get(self, "private_ips")

    @_builtins.property
    @pulumi.getter(name="publicIp")
    def public_ip(self) -> _builtins.str:
        """
        The public IPv4.
        """
        return pulumi.get(self, "public_ip")

    @_builtins.property
    @pulumi.getter(name="publicIpV6")
    def public_ip_v6(self) -> _builtins.str:
        """
        The public IPv6.
        """
        return pulumi.get(self, "public_ip_v6")

    @_builtins.property
    @pulumi.getter
    def status(self) -> _builtins.str:
        """
        The status of the node.
        """
        return pulumi.get(self, "status")


@pulumi.output_type
class GetPoolNodePrivateIpResult(dict):
    def __init__(__self__, *,
                 address: _builtins.str,
                 id: _builtins.str):
        """
        :param _builtins.str address: The private IP address
        :param _builtins.str id: The ID of the pool.
        """
        pulumi.set(__self__, "address", address)
        pulumi.set(__self__, "id", id)

    @_builtins.property
    @pulumi.getter
    def address(self) -> _builtins.str:
        """
        The private IP address
        """
        return pulumi.get(self, "address")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The ID of the pool.
        """
        return pulumi.get(self, "id")


@pulumi.output_type
class GetPoolUpgradePolicyResult(dict):
    def __init__(__self__, *,
                 max_surge: _builtins.int,
                 max_unavailable: _builtins.int):
        """
        :param _builtins.int max_surge: The maximum number of nodes to be created during the upgrade
        :param _builtins.int max_unavailable: The maximum number of nodes that can be not ready at the same time
        """
        pulumi.set(__self__, "max_surge", max_surge)
        pulumi.set(__self__, "max_unavailable", max_unavailable)

    @_builtins.property
    @pulumi.getter(name="maxSurge")
    def max_surge(self) -> _builtins.int:
        """
        The maximum number of nodes to be created during the upgrade
        """
        return pulumi.get(self, "max_surge")

    @_builtins.property
    @pulumi.getter(name="maxUnavailable")
    def max_unavailable(self) -> _builtins.int:
        """
        The maximum number of nodes that can be not ready at the same time
        """
        return pulumi.get(self, "max_unavailable")





# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['DatabaseAclArgs', 'DatabaseAcl']

@pulumi.input_type
class DatabaseAclArgs:
    def __init__(__self__, *,
                 acl_rules: pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]],
                 instance_id: pulumi.Input[_builtins.str],
                 region: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a DatabaseAcl resource.
        :param pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]] acl_rules: A list of ACLs (structure is described below)
               
               > **Important:** The `databases.Acl` resource replaces **all** ACL rules for the given instance. Multiple `databases.Acl` resources targeting the same `instance_id` will conflict with each other. Use multiple `acl_rules` blocks within a single resource instead.
        :param pulumi.Input[_builtins.str] instance_id: UUID of the Database Instance.
               
               > **Important:** Updates to `instance_id` will recreate the Database ACL.
        :param pulumi.Input[_builtins.str] region: `region`) The region in which the Database Instance should be created.
        """
        pulumi.set(__self__, "acl_rules", acl_rules)
        pulumi.set(__self__, "instance_id", instance_id)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter(name="aclRules")
    def acl_rules(self) -> pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]]:
        """
        A list of ACLs (structure is described below)

        > **Important:** The `databases.Acl` resource replaces **all** ACL rules for the given instance. Multiple `databases.Acl` resources targeting the same `instance_id` will conflict with each other. Use multiple `acl_rules` blocks within a single resource instead.
        """
        return pulumi.get(self, "acl_rules")

    @acl_rules.setter
    def acl_rules(self, value: pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]]):
        pulumi.set(self, "acl_rules", value)

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Input[_builtins.str]:
        """
        UUID of the Database Instance.

        > **Important:** Updates to `instance_id` will recreate the Database ACL.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "instance_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `region`) The region in which the Database Instance should be created.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class _DatabaseAclState:
    def __init__(__self__, *,
                 acl_rules: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering DatabaseAcl resources.
        :param pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]] acl_rules: A list of ACLs (structure is described below)
               
               > **Important:** The `databases.Acl` resource replaces **all** ACL rules for the given instance. Multiple `databases.Acl` resources targeting the same `instance_id` will conflict with each other. Use multiple `acl_rules` blocks within a single resource instead.
        :param pulumi.Input[_builtins.str] instance_id: UUID of the Database Instance.
               
               > **Important:** Updates to `instance_id` will recreate the Database ACL.
        :param pulumi.Input[_builtins.str] region: `region`) The region in which the Database Instance should be created.
        """
        if acl_rules is not None:
            pulumi.set(__self__, "acl_rules", acl_rules)
        if instance_id is not None:
            pulumi.set(__self__, "instance_id", instance_id)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter(name="aclRules")
    def acl_rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]]]:
        """
        A list of ACLs (structure is described below)

        > **Important:** The `databases.Acl` resource replaces **all** ACL rules for the given instance. Multiple `databases.Acl` resources targeting the same `instance_id` will conflict with each other. Use multiple `acl_rules` blocks within a single resource instead.
        """
        return pulumi.get(self, "acl_rules")

    @acl_rules.setter
    def acl_rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseAclAclRuleArgs']]]]):
        pulumi.set(self, "acl_rules", value)

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        UUID of the Database Instance.

        > **Important:** Updates to `instance_id` will recreate the Database ACL.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "instance_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `region`) The region in which the Database Instance should be created.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)


warnings.warn("""scaleway.index/databaseacl.DatabaseAcl has been deprecated in favor of scaleway.databases/acl.Acl""", DeprecationWarning)


@pulumi.type_token("scaleway:index/databaseAcl:DatabaseAcl")
class DatabaseAcl(pulumi.CustomResource):
    warnings.warn("""scaleway.index/databaseacl.DatabaseAcl has been deprecated in favor of scaleway.databases/acl.Acl""", DeprecationWarning)

    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 acl_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseAclAclRuleArgs', 'DatabaseAclAclRuleArgsDict']]]]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Creates and manages Scaleway Database instance authorized IPs.
        For more information refer to the [API documentation](https://www.scaleway.com/en/developers/api/managed-database-postgre-mysql/#acl-rules-allowed-ips).

        ## Example Usage

        ### Basic

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        main = scaleway.databases.Instance("main",
            name="test-rdb",
            node_type="DB-DEV-S",
            engine="PostgreSQL-15",
            is_ha_cluster=True,
            disable_backup=True,
            user_name="my_initial_user",
            password="thiZ_is_v&ry_s3cret")
        main_acl = scaleway.databases.Acl("main",
            instance_id=main.id,
            acl_rules=[{
                "ip": "1.2.3.4/32",
                "description": "foo",
            }])
        ```

        ### Multiple ACL Rules

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        main = scaleway.databases.Acl("main",
            instance_id=main_scaleway_rdb_instance["id"],
            acl_rules=[
                {
                    "ip": "1.2.3.4/32",
                    "description": "Office IP",
                },
                {
                    "ip": "5.6.7.8/32",
                    "description": "Home IP",
                },
                {
                    "ip": "10.0.0.0/24",
                    "description": "Internal network",
                },
            ])
        ```

        ### Dynamic ACL Rules with Variables

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        config = pulumi.Config()
        # Map of allowed IPs with descriptions
        allowed_ips = config.get_object("allowedIps")
        if allowed_ips is None:
            allowed_ips = {
                "1.2.3.4/32": "Office IP",
                "10.0.0.0/24": "Internal network",
                "5.6.7.8/32": "Home IP",
            }
        main = scaleway.databases.Acl("main",
            acl_rules=[{
                "ip": entry["key"],
                "description": entry["value"],
            } for entry in [{"key": k, "value": v} for k, v in allowed_ips]],
            instance_id=main_scaleway_rdb_instance["id"])
        ```

        ## Import

        Database Instance can be imported using the `{region}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:index/databaseAcl:DatabaseAcl acl01 fr-par/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['DatabaseAclAclRuleArgs', 'DatabaseAclAclRuleArgsDict']]]] acl_rules: A list of ACLs (structure is described below)
               
               > **Important:** The `databases.Acl` resource replaces **all** ACL rules for the given instance. Multiple `databases.Acl` resources targeting the same `instance_id` will conflict with each other. Use multiple `acl_rules` blocks within a single resource instead.
        :param pulumi.Input[_builtins.str] instance_id: UUID of the Database Instance.
               
               > **Important:** Updates to `instance_id` will recreate the Database ACL.
        :param pulumi.Input[_builtins.str] region: `region`) The region in which the Database Instance should be created.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: DatabaseAclArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Creates and manages Scaleway Database instance authorized IPs.
        For more information refer to the [API documentation](https://www.scaleway.com/en/developers/api/managed-database-postgre-mysql/#acl-rules-allowed-ips).

        ## Example Usage

        ### Basic

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        main = scaleway.databases.Instance("main",
            name="test-rdb",
            node_type="DB-DEV-S",
            engine="PostgreSQL-15",
            is_ha_cluster=True,
            disable_backup=True,
            user_name="my_initial_user",
            password="thiZ_is_v&ry_s3cret")
        main_acl = scaleway.databases.Acl("main",
            instance_id=main.id,
            acl_rules=[{
                "ip": "1.2.3.4/32",
                "description": "foo",
            }])
        ```

        ### Multiple ACL Rules

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        main = scaleway.databases.Acl("main",
            instance_id=main_scaleway_rdb_instance["id"],
            acl_rules=[
                {
                    "ip": "1.2.3.4/32",
                    "description": "Office IP",
                },
                {
                    "ip": "5.6.7.8/32",
                    "description": "Home IP",
                },
                {
                    "ip": "10.0.0.0/24",
                    "description": "Internal network",
                },
            ])
        ```

        ### Dynamic ACL Rules with Variables

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        config = pulumi.Config()
        # Map of allowed IPs with descriptions
        allowed_ips = config.get_object("allowedIps")
        if allowed_ips is None:
            allowed_ips = {
                "1.2.3.4/32": "Office IP",
                "10.0.0.0/24": "Internal network",
                "5.6.7.8/32": "Home IP",
            }
        main = scaleway.databases.Acl("main",
            acl_rules=[{
                "ip": entry["key"],
                "description": entry["value"],
            } for entry in [{"key": k, "value": v} for k, v in allowed_ips]],
            instance_id=main_scaleway_rdb_instance["id"])
        ```

        ## Import

        Database Instance can be imported using the `{region}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:index/databaseAcl:DatabaseAcl acl01 fr-par/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param DatabaseAclArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(DatabaseAclArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 acl_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseAclAclRuleArgs', 'DatabaseAclAclRuleArgsDict']]]]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        pulumi.log.warn("""DatabaseAcl is deprecated: scaleway.index/databaseacl.DatabaseAcl has been deprecated in favor of scaleway.databases/acl.Acl""")
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = DatabaseAclArgs.__new__(DatabaseAclArgs)

            if acl_rules is None and not opts.urn:
                raise TypeError("Missing required property 'acl_rules'")
            __props__.__dict__["acl_rules"] = acl_rules
            if instance_id is None and not opts.urn:
                raise TypeError("Missing required property 'instance_id'")
            __props__.__dict__["instance_id"] = instance_id
            __props__.__dict__["region"] = region
        super(DatabaseAcl, __self__).__init__(
            'scaleway:index/databaseAcl:DatabaseAcl',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            acl_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseAclAclRuleArgs', 'DatabaseAclAclRuleArgsDict']]]]] = None,
            instance_id: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None) -> 'DatabaseAcl':
        """
        Get an existing DatabaseAcl resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['DatabaseAclAclRuleArgs', 'DatabaseAclAclRuleArgsDict']]]] acl_rules: A list of ACLs (structure is described below)
               
               > **Important:** The `databases.Acl` resource replaces **all** ACL rules for the given instance. Multiple `databases.Acl` resources targeting the same `instance_id` will conflict with each other. Use multiple `acl_rules` blocks within a single resource instead.
        :param pulumi.Input[_builtins.str] instance_id: UUID of the Database Instance.
               
               > **Important:** Updates to `instance_id` will recreate the Database ACL.
        :param pulumi.Input[_builtins.str] region: `region`) The region in which the Database Instance should be created.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _DatabaseAclState.__new__(_DatabaseAclState)

        __props__.__dict__["acl_rules"] = acl_rules
        __props__.__dict__["instance_id"] = instance_id
        __props__.__dict__["region"] = region
        return DatabaseAcl(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="aclRules")
    def acl_rules(self) -> pulumi.Output[Sequence['outputs.DatabaseAclAclRule']]:
        """
        A list of ACLs (structure is described below)

        > **Important:** The `databases.Acl` resource replaces **all** ACL rules for the given instance. Multiple `databases.Acl` resources targeting the same `instance_id` will conflict with each other. Use multiple `acl_rules` blocks within a single resource instead.
        """
        return pulumi.get(self, "acl_rules")

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Output[_builtins.str]:
        """
        UUID of the Database Instance.

        > **Important:** Updates to `instance_id` will recreate the Database ACL.
        """
        return pulumi.get(self, "instance_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        `region`) The region in which the Database Instance should be created.
        """
        return pulumi.get(self, "region")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetLbFrontendResult',
    'AwaitableGetLbFrontendResult',
    'get_lb_frontend',
    'get_lb_frontend_output',
]

warnings.warn("""scaleway.index/getlbfrontend.getLbFrontend has been deprecated in favor of scaleway.loadbalancers/getfrontend.getFrontend""", DeprecationWarning)

@pulumi.output_type
class GetLbFrontendResult:
    """
    A collection of values returned by getLbFrontend.
    """
    def __init__(__self__, acls=None, backend_id=None, certificate_id=None, certificate_ids=None, connection_rate_limit=None, created_at=None, enable_access_logs=None, enable_http3=None, external_acls=None, frontend_id=None, id=None, inbound_port=None, lb_id=None, name=None, timeout_client=None, updated_at=None):
        if acls and not isinstance(acls, list):
            raise TypeError("Expected argument 'acls' to be a list")
        pulumi.set(__self__, "acls", acls)
        if backend_id and not isinstance(backend_id, str):
            raise TypeError("Expected argument 'backend_id' to be a str")
        pulumi.set(__self__, "backend_id", backend_id)
        if certificate_id and not isinstance(certificate_id, str):
            raise TypeError("Expected argument 'certificate_id' to be a str")
        pulumi.set(__self__, "certificate_id", certificate_id)
        if certificate_ids and not isinstance(certificate_ids, list):
            raise TypeError("Expected argument 'certificate_ids' to be a list")
        pulumi.set(__self__, "certificate_ids", certificate_ids)
        if connection_rate_limit and not isinstance(connection_rate_limit, int):
            raise TypeError("Expected argument 'connection_rate_limit' to be a int")
        pulumi.set(__self__, "connection_rate_limit", connection_rate_limit)
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if enable_access_logs and not isinstance(enable_access_logs, bool):
            raise TypeError("Expected argument 'enable_access_logs' to be a bool")
        pulumi.set(__self__, "enable_access_logs", enable_access_logs)
        if enable_http3 and not isinstance(enable_http3, bool):
            raise TypeError("Expected argument 'enable_http3' to be a bool")
        pulumi.set(__self__, "enable_http3", enable_http3)
        if external_acls and not isinstance(external_acls, bool):
            raise TypeError("Expected argument 'external_acls' to be a bool")
        pulumi.set(__self__, "external_acls", external_acls)
        if frontend_id and not isinstance(frontend_id, str):
            raise TypeError("Expected argument 'frontend_id' to be a str")
        pulumi.set(__self__, "frontend_id", frontend_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if inbound_port and not isinstance(inbound_port, int):
            raise TypeError("Expected argument 'inbound_port' to be a int")
        pulumi.set(__self__, "inbound_port", inbound_port)
        if lb_id and not isinstance(lb_id, str):
            raise TypeError("Expected argument 'lb_id' to be a str")
        pulumi.set(__self__, "lb_id", lb_id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if timeout_client and not isinstance(timeout_client, str):
            raise TypeError("Expected argument 'timeout_client' to be a str")
        pulumi.set(__self__, "timeout_client", timeout_client)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)

    @_builtins.property
    @pulumi.getter
    def acls(self) -> Sequence['outputs.GetLbFrontendAclResult']:
        return pulumi.get(self, "acls")

    @_builtins.property
    @pulumi.getter(name="backendId")
    def backend_id(self) -> _builtins.str:
        return pulumi.get(self, "backend_id")

    @_builtins.property
    @pulumi.getter(name="certificateId")
    def certificate_id(self) -> _builtins.str:
        return pulumi.get(self, "certificate_id")

    @_builtins.property
    @pulumi.getter(name="certificateIds")
    def certificate_ids(self) -> Sequence[_builtins.str]:
        return pulumi.get(self, "certificate_ids")

    @_builtins.property
    @pulumi.getter(name="connectionRateLimit")
    def connection_rate_limit(self) -> _builtins.int:
        return pulumi.get(self, "connection_rate_limit")

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> _builtins.str:
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter(name="enableAccessLogs")
    def enable_access_logs(self) -> _builtins.bool:
        return pulumi.get(self, "enable_access_logs")

    @_builtins.property
    @pulumi.getter(name="enableHttp3")
    def enable_http3(self) -> _builtins.bool:
        return pulumi.get(self, "enable_http3")

    @_builtins.property
    @pulumi.getter(name="externalAcls")
    def external_acls(self) -> _builtins.bool:
        return pulumi.get(self, "external_acls")

    @_builtins.property
    @pulumi.getter(name="frontendId")
    def frontend_id(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "frontend_id")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="inboundPort")
    def inbound_port(self) -> _builtins.int:
        return pulumi.get(self, "inbound_port")

    @_builtins.property
    @pulumi.getter(name="lbId")
    def lb_id(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "lb_id")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="timeoutClient")
    def timeout_client(self) -> _builtins.str:
        return pulumi.get(self, "timeout_client")

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> _builtins.str:
        return pulumi.get(self, "updated_at")


class AwaitableGetLbFrontendResult(GetLbFrontendResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetLbFrontendResult(
            acls=self.acls,
            backend_id=self.backend_id,
            certificate_id=self.certificate_id,
            certificate_ids=self.certificate_ids,
            connection_rate_limit=self.connection_rate_limit,
            created_at=self.created_at,
            enable_access_logs=self.enable_access_logs,
            enable_http3=self.enable_http3,
            external_acls=self.external_acls,
            frontend_id=self.frontend_id,
            id=self.id,
            inbound_port=self.inbound_port,
            lb_id=self.lb_id,
            name=self.name,
            timeout_client=self.timeout_client,
            updated_at=self.updated_at)


def get_lb_frontend(frontend_id: Optional[_builtins.str] = None,
                    lb_id: Optional[_builtins.str] = None,
                    name: Optional[_builtins.str] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetLbFrontendResult:
    """
    Get information about Scaleway Load Balancer frontends.

    For more information, see the [main documentation](https://www.scaleway.com/en/docs/load-balancer/reference-content/configuring-frontends/) or [API documentation](https://www.scaleway.com/en/developers/api/load-balancer/zoned-api/#path-frontends).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_scaleway as scaleway
    import pulumiverse_scaleway as scaleway

    ip01 = scaleway.loadbalancers.Ip("ip01")
    lb01 = scaleway.loadbalancers.LoadBalancer("lb01",
        ip_id=ip01.id,
        name="test-lb",
        type="lb-s")
    bkd01 = scaleway.loadbalancers.Backend("bkd01",
        lb_id=lb01.id,
        forward_protocol="tcp",
        forward_port=80,
        proxy_protocol="none")
    frt01 = scaleway.loadbalancers.Frontend("frt01",
        lb_id=lb01.id,
        backend_id=bkd01.id,
        inbound_port=80)
    by_id = scaleway.loadbalancers.get_frontend_output(frontend_id=frt01.id)
    by_name = scaleway.loadbalancers.get_frontend_output(name=frt01.name,
        lb_id=lb01.id)
    ```


    :param _builtins.str frontend_id: The frontend ID.
           - Only one of `name` and `frontend_id` should be specified.
    :param _builtins.str lb_id: The Load Balancer ID this frontend is attached to.
    :param _builtins.str name: The name of the frontend.
           - When using the `name` you should specify the `lb-id`
    """
    pulumi.log.warn("""get_lb_frontend is deprecated: scaleway.index/getlbfrontend.getLbFrontend has been deprecated in favor of scaleway.loadbalancers/getfrontend.getFrontend""")
    __args__ = dict()
    __args__['frontendId'] = frontend_id
    __args__['lbId'] = lb_id
    __args__['name'] = name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('scaleway:index/getLbFrontend:getLbFrontend', __args__, opts=opts, typ=GetLbFrontendResult).value

    return AwaitableGetLbFrontendResult(
        acls=pulumi.get(__ret__, 'acls'),
        backend_id=pulumi.get(__ret__, 'backend_id'),
        certificate_id=pulumi.get(__ret__, 'certificate_id'),
        certificate_ids=pulumi.get(__ret__, 'certificate_ids'),
        connection_rate_limit=pulumi.get(__ret__, 'connection_rate_limit'),
        created_at=pulumi.get(__ret__, 'created_at'),
        enable_access_logs=pulumi.get(__ret__, 'enable_access_logs'),
        enable_http3=pulumi.get(__ret__, 'enable_http3'),
        external_acls=pulumi.get(__ret__, 'external_acls'),
        frontend_id=pulumi.get(__ret__, 'frontend_id'),
        id=pulumi.get(__ret__, 'id'),
        inbound_port=pulumi.get(__ret__, 'inbound_port'),
        lb_id=pulumi.get(__ret__, 'lb_id'),
        name=pulumi.get(__ret__, 'name'),
        timeout_client=pulumi.get(__ret__, 'timeout_client'),
        updated_at=pulumi.get(__ret__, 'updated_at'))
def get_lb_frontend_output(frontend_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                           lb_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                           name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                           opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetLbFrontendResult]:
    """
    Get information about Scaleway Load Balancer frontends.

    For more information, see the [main documentation](https://www.scaleway.com/en/docs/load-balancer/reference-content/configuring-frontends/) or [API documentation](https://www.scaleway.com/en/developers/api/load-balancer/zoned-api/#path-frontends).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_scaleway as scaleway
    import pulumiverse_scaleway as scaleway

    ip01 = scaleway.loadbalancers.Ip("ip01")
    lb01 = scaleway.loadbalancers.LoadBalancer("lb01",
        ip_id=ip01.id,
        name="test-lb",
        type="lb-s")
    bkd01 = scaleway.loadbalancers.Backend("bkd01",
        lb_id=lb01.id,
        forward_protocol="tcp",
        forward_port=80,
        proxy_protocol="none")
    frt01 = scaleway.loadbalancers.Frontend("frt01",
        lb_id=lb01.id,
        backend_id=bkd01.id,
        inbound_port=80)
    by_id = scaleway.loadbalancers.get_frontend_output(frontend_id=frt01.id)
    by_name = scaleway.loadbalancers.get_frontend_output(name=frt01.name,
        lb_id=lb01.id)
    ```


    :param _builtins.str frontend_id: The frontend ID.
           - Only one of `name` and `frontend_id` should be specified.
    :param _builtins.str lb_id: The Load Balancer ID this frontend is attached to.
    :param _builtins.str name: The name of the frontend.
           - When using the `name` you should specify the `lb-id`
    """
    pulumi.log.warn("""get_lb_frontend is deprecated: scaleway.index/getlbfrontend.getLbFrontend has been deprecated in favor of scaleway.loadbalancers/getfrontend.getFrontend""")
    __args__ = dict()
    __args__['frontendId'] = frontend_id
    __args__['lbId'] = lb_id
    __args__['name'] = name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('scaleway:index/getLbFrontend:getLbFrontend', __args__, opts=opts, typ=GetLbFrontendResult)
    return __ret__.apply(lambda __response__: GetLbFrontendResult(
        acls=pulumi.get(__response__, 'acls'),
        backend_id=pulumi.get(__response__, 'backend_id'),
        certificate_id=pulumi.get(__response__, 'certificate_id'),
        certificate_ids=pulumi.get(__response__, 'certificate_ids'),
        connection_rate_limit=pulumi.get(__response__, 'connection_rate_limit'),
        created_at=pulumi.get(__response__, 'created_at'),
        enable_access_logs=pulumi.get(__response__, 'enable_access_logs'),
        enable_http3=pulumi.get(__response__, 'enable_http3'),
        external_acls=pulumi.get(__response__, 'external_acls'),
        frontend_id=pulumi.get(__response__, 'frontend_id'),
        id=pulumi.get(__response__, 'id'),
        inbound_port=pulumi.get(__response__, 'inbound_port'),
        lb_id=pulumi.get(__response__, 'lb_id'),
        name=pulumi.get(__response__, 'name'),
        timeout_client=pulumi.get(__response__, 'timeout_client'),
        updated_at=pulumi.get(__response__, 'updated_at')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'DeploymentPrivateNetworkArgs',
    'DeploymentPrivateNetworkArgsDict',
    'DeploymentPrivateNetworkServiceArgs',
    'DeploymentPrivateNetworkServiceArgsDict',
    'DeploymentPublicNetworkArgs',
    'DeploymentPublicNetworkArgsDict',
    'DeploymentPublicNetworkServiceArgs',
    'DeploymentPublicNetworkServiceArgsDict',
]

MYPY = False

if not MYPY:
    class DeploymentPrivateNetworkArgsDict(TypedDict):
        pn_id: pulumi.Input[_builtins.str]
        """
        The ID of the private network. Format: `{region}/{id}` or just `{id}`.
        """
        dns_record: NotRequired[pulumi.Input[_builtins.str]]
        """
        DNS record for the private endpoint.
        """
        id: NotRequired[pulumi.Input[_builtins.str]]
        """
        The ID of the private endpoint.
        """
        services: NotRequired[pulumi.Input[Sequence[pulumi.Input['DeploymentPrivateNetworkServiceArgsDict']]]]
        """
        List of services exposed on the private endpoint.
        """
elif False:
    DeploymentPrivateNetworkArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class DeploymentPrivateNetworkArgs:
    def __init__(__self__, *,
                 pn_id: pulumi.Input[_builtins.str],
                 dns_record: Optional[pulumi.Input[_builtins.str]] = None,
                 id: Optional[pulumi.Input[_builtins.str]] = None,
                 services: Optional[pulumi.Input[Sequence[pulumi.Input['DeploymentPrivateNetworkServiceArgs']]]] = None):
        """
        :param pulumi.Input[_builtins.str] pn_id: The ID of the private network. Format: `{region}/{id}` or just `{id}`.
        :param pulumi.Input[_builtins.str] dns_record: DNS record for the private endpoint.
        :param pulumi.Input[_builtins.str] id: The ID of the private endpoint.
        :param pulumi.Input[Sequence[pulumi.Input['DeploymentPrivateNetworkServiceArgs']]] services: List of services exposed on the private endpoint.
        """
        pulumi.set(__self__, "pn_id", pn_id)
        if dns_record is not None:
            pulumi.set(__self__, "dns_record", dns_record)
        if id is not None:
            pulumi.set(__self__, "id", id)
        if services is not None:
            pulumi.set(__self__, "services", services)

    @_builtins.property
    @pulumi.getter(name="pnId")
    def pn_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the private network. Format: `{region}/{id}` or just `{id}`.
        """
        return pulumi.get(self, "pn_id")

    @pn_id.setter
    def pn_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "pn_id", value)

    @_builtins.property
    @pulumi.getter(name="dnsRecord")
    def dns_record(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        DNS record for the private endpoint.
        """
        return pulumi.get(self, "dns_record")

    @dns_record.setter
    def dns_record(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "dns_record", value)

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the private endpoint.
        """
        return pulumi.get(self, "id")

    @id.setter
    def id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "id", value)

    @_builtins.property
    @pulumi.getter
    def services(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DeploymentPrivateNetworkServiceArgs']]]]:
        """
        List of services exposed on the private endpoint.
        """
        return pulumi.get(self, "services")

    @services.setter
    def services(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DeploymentPrivateNetworkServiceArgs']]]]):
        pulumi.set(self, "services", value)


if not MYPY:
    class DeploymentPrivateNetworkServiceArgsDict(TypedDict):
        port: NotRequired[pulumi.Input[_builtins.int]]
        """
        TCP port number.
        """
        protocol: NotRequired[pulumi.Input[_builtins.str]]
        """
        Service protocol (e.g., "tcp", "https", "mysql").
        """
elif False:
    DeploymentPrivateNetworkServiceArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class DeploymentPrivateNetworkServiceArgs:
    def __init__(__self__, *,
                 port: Optional[pulumi.Input[_builtins.int]] = None,
                 protocol: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.int] port: TCP port number.
        :param pulumi.Input[_builtins.str] protocol: Service protocol (e.g., "tcp", "https", "mysql").
        """
        if port is not None:
            pulumi.set(__self__, "port", port)
        if protocol is not None:
            pulumi.set(__self__, "protocol", protocol)

    @_builtins.property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        TCP port number.
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "port", value)

    @_builtins.property
    @pulumi.getter
    def protocol(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Service protocol (e.g., "tcp", "https", "mysql").
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "protocol", value)


if not MYPY:
    class DeploymentPublicNetworkArgsDict(TypedDict):
        dns_record: NotRequired[pulumi.Input[_builtins.str]]
        """
        DNS record for the private endpoint.
        """
        id: NotRequired[pulumi.Input[_builtins.str]]
        """
        The ID of the private endpoint.
        """
        services: NotRequired[pulumi.Input[Sequence[pulumi.Input['DeploymentPublicNetworkServiceArgsDict']]]]
        """
        List of services exposed on the private endpoint.
        """
elif False:
    DeploymentPublicNetworkArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class DeploymentPublicNetworkArgs:
    def __init__(__self__, *,
                 dns_record: Optional[pulumi.Input[_builtins.str]] = None,
                 id: Optional[pulumi.Input[_builtins.str]] = None,
                 services: Optional[pulumi.Input[Sequence[pulumi.Input['DeploymentPublicNetworkServiceArgs']]]] = None):
        """
        :param pulumi.Input[_builtins.str] dns_record: DNS record for the private endpoint.
        :param pulumi.Input[_builtins.str] id: The ID of the private endpoint.
        :param pulumi.Input[Sequence[pulumi.Input['DeploymentPublicNetworkServiceArgs']]] services: List of services exposed on the private endpoint.
        """
        if dns_record is not None:
            pulumi.set(__self__, "dns_record", dns_record)
        if id is not None:
            pulumi.set(__self__, "id", id)
        if services is not None:
            pulumi.set(__self__, "services", services)

    @_builtins.property
    @pulumi.getter(name="dnsRecord")
    def dns_record(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        DNS record for the private endpoint.
        """
        return pulumi.get(self, "dns_record")

    @dns_record.setter
    def dns_record(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "dns_record", value)

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the private endpoint.
        """
        return pulumi.get(self, "id")

    @id.setter
    def id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "id", value)

    @_builtins.property
    @pulumi.getter
    def services(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DeploymentPublicNetworkServiceArgs']]]]:
        """
        List of services exposed on the private endpoint.
        """
        return pulumi.get(self, "services")

    @services.setter
    def services(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DeploymentPublicNetworkServiceArgs']]]]):
        pulumi.set(self, "services", value)


if not MYPY:
    class DeploymentPublicNetworkServiceArgsDict(TypedDict):
        port: NotRequired[pulumi.Input[_builtins.int]]
        """
        TCP port number.
        """
        protocol: NotRequired[pulumi.Input[_builtins.str]]
        """
        Service protocol (e.g., "tcp", "https", "mysql").
        """
elif False:
    DeploymentPublicNetworkServiceArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class DeploymentPublicNetworkServiceArgs:
    def __init__(__self__, *,
                 port: Optional[pulumi.Input[_builtins.int]] = None,
                 protocol: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.int] port: TCP port number.
        :param pulumi.Input[_builtins.str] protocol: Service protocol (e.g., "tcp", "https", "mysql").
        """
        if port is not None:
            pulumi.set(__self__, "port", port)
        if protocol is not None:
            pulumi.set(__self__, "protocol", protocol)

    @_builtins.property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        TCP port number.
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "port", value)

    @_builtins.property
    @pulumi.getter
    def protocol(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Service protocol (e.g., "tcp", "https", "mysql").
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "protocol", value)





# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'ServerPrivateIp',
    'ServerPrivateNetwork',
]

@pulumi.output_type
class ServerPrivateIp(dict):
    def __init__(__self__, *,
                 address: Optional[_builtins.str] = None,
                 id: Optional[_builtins.str] = None):
        """
        :param _builtins.str address: The private IP address.
        :param _builtins.str id: The ID of the IP address resource.
        """
        if address is not None:
            pulumi.set(__self__, "address", address)
        if id is not None:
            pulumi.set(__self__, "id", id)

    @_builtins.property
    @pulumi.getter
    def address(self) -> Optional[_builtins.str]:
        """
        The private IP address.
        """
        return pulumi.get(self, "address")

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[_builtins.str]:
        """
        The ID of the IP address resource.
        """
        return pulumi.get(self, "id")


@pulumi.output_type
class ServerPrivateNetwork(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "createdAt":
            suggest = "created_at"
        elif key == "ipamIpIds":
            suggest = "ipam_ip_ids"
        elif key == "updatedAt":
            suggest = "updated_at"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ServerPrivateNetwork. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ServerPrivateNetwork.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ServerPrivateNetwork.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 id: _builtins.str,
                 created_at: Optional[_builtins.str] = None,
                 ipam_ip_ids: Optional[Sequence[_builtins.str]] = None,
                 status: Optional[_builtins.str] = None,
                 updated_at: Optional[_builtins.str] = None,
                 vlan: Optional[_builtins.int] = None):
        """
        :param _builtins.str id: The private network ID
        :param _builtins.str created_at: The date and time the private network was created.
        :param Sequence[_builtins.str] ipam_ip_ids: A list of IPAM IP IDs to attach to the server.
        :param _builtins.str status: The current status of the private network.
        :param _builtins.str updated_at: The date and time the private network was last updated.
        :param _builtins.int vlan: The VLAN ID associated with the private network.
        """
        pulumi.set(__self__, "id", id)
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if ipam_ip_ids is not None:
            pulumi.set(__self__, "ipam_ip_ids", ipam_ip_ids)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if updated_at is not None:
            pulumi.set(__self__, "updated_at", updated_at)
        if vlan is not None:
            pulumi.set(__self__, "vlan", vlan)

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The private network ID
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[_builtins.str]:
        """
        The date and time the private network was created.
        """
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter(name="ipamIpIds")
    def ipam_ip_ids(self) -> Optional[Sequence[_builtins.str]]:
        """
        A list of IPAM IP IDs to attach to the server.
        """
        return pulumi.get(self, "ipam_ip_ids")

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[_builtins.str]:
        """
        The current status of the private network.
        """
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> Optional[_builtins.str]:
        """
        The date and time the private network was last updated.
        """
        return pulumi.get(self, "updated_at")

    @_builtins.property
    @pulumi.getter
    def vlan(self) -> Optional[_builtins.int]:
        """
        The VLAN ID associated with the private network.
        """
        return pulumi.get(self, "vlan")





# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['RouteArgs', 'Route']

@pulumi.input_type
class RouteArgs:
    def __init__(__self__, *,
                 vpc_id: pulumi.Input[_builtins.str],
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 destination: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_private_network_id: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_resource_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        The set of arguments for constructing a Route resource.
        :param pulumi.Input[_builtins.str] vpc_id: The VPC ID the route belongs to.
        :param pulumi.Input[_builtins.str] description: The route description.
        :param pulumi.Input[_builtins.str] destination: The destination of the route.
        :param pulumi.Input[_builtins.str] nexthop_private_network_id: The ID of the nexthop private network.
        :param pulumi.Input[_builtins.str] nexthop_resource_id: The ID of the nexthop resource.
        :param pulumi.Input[_builtins.str] region: `region`) The region of the route.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags to associate with the route.
        """
        pulumi.set(__self__, "vpc_id", vpc_id)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if destination is not None:
            pulumi.set(__self__, "destination", destination)
        if nexthop_private_network_id is not None:
            pulumi.set(__self__, "nexthop_private_network_id", nexthop_private_network_id)
        if nexthop_resource_id is not None:
            pulumi.set(__self__, "nexthop_resource_id", nexthop_resource_id)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> pulumi.Input[_builtins.str]:
        """
        The VPC ID the route belongs to.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "vpc_id", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The route description.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter
    def destination(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The destination of the route.
        """
        return pulumi.get(self, "destination")

    @destination.setter
    def destination(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "destination", value)

    @_builtins.property
    @pulumi.getter(name="nexthopPrivateNetworkId")
    def nexthop_private_network_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the nexthop private network.
        """
        return pulumi.get(self, "nexthop_private_network_id")

    @nexthop_private_network_id.setter
    def nexthop_private_network_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "nexthop_private_network_id", value)

    @_builtins.property
    @pulumi.getter(name="nexthopResourceId")
    def nexthop_resource_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the nexthop resource.
        """
        return pulumi.get(self, "nexthop_resource_id")

    @nexthop_resource_id.setter
    def nexthop_resource_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "nexthop_resource_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `region`) The region of the route.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The tags to associate with the route.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _RouteState:
    def __init__(__self__, *,
                 created_at: Optional[pulumi.Input[_builtins.str]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 destination: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_private_network_id: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_resource_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 updated_at: Optional[pulumi.Input[_builtins.str]] = None,
                 vpc_id: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering Route resources.
        :param pulumi.Input[_builtins.str] created_at: The date and time of the creation of the route (RFC 3339 format).
        :param pulumi.Input[_builtins.str] description: The route description.
        :param pulumi.Input[_builtins.str] destination: The destination of the route.
        :param pulumi.Input[_builtins.str] nexthop_private_network_id: The ID of the nexthop private network.
        :param pulumi.Input[_builtins.str] nexthop_resource_id: The ID of the nexthop resource.
        :param pulumi.Input[_builtins.str] region: `region`) The region of the route.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags to associate with the route.
        :param pulumi.Input[_builtins.str] updated_at: The date and time of the creation of the route (RFC 3339 format).
        :param pulumi.Input[_builtins.str] vpc_id: The VPC ID the route belongs to.
        """
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if destination is not None:
            pulumi.set(__self__, "destination", destination)
        if nexthop_private_network_id is not None:
            pulumi.set(__self__, "nexthop_private_network_id", nexthop_private_network_id)
        if nexthop_resource_id is not None:
            pulumi.set(__self__, "nexthop_resource_id", nexthop_resource_id)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if updated_at is not None:
            pulumi.set(__self__, "updated_at", updated_at)
        if vpc_id is not None:
            pulumi.set(__self__, "vpc_id", vpc_id)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The date and time of the creation of the route (RFC 3339 format).
        """
        return pulumi.get(self, "created_at")

    @created_at.setter
    def created_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "created_at", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The route description.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter
    def destination(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The destination of the route.
        """
        return pulumi.get(self, "destination")

    @destination.setter
    def destination(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "destination", value)

    @_builtins.property
    @pulumi.getter(name="nexthopPrivateNetworkId")
    def nexthop_private_network_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the nexthop private network.
        """
        return pulumi.get(self, "nexthop_private_network_id")

    @nexthop_private_network_id.setter
    def nexthop_private_network_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "nexthop_private_network_id", value)

    @_builtins.property
    @pulumi.getter(name="nexthopResourceId")
    def nexthop_resource_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the nexthop resource.
        """
        return pulumi.get(self, "nexthop_resource_id")

    @nexthop_resource_id.setter
    def nexthop_resource_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "nexthop_resource_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `region`) The region of the route.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The tags to associate with the route.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The date and time of the creation of the route (RFC 3339 format).
        """
        return pulumi.get(self, "updated_at")

    @updated_at.setter
    def updated_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "updated_at", value)

    @_builtins.property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The VPC ID the route belongs to.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "vpc_id", value)


@pulumi.type_token("scaleway:network/route:Route")
class Route(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 destination: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_private_network_id: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_resource_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 vpc_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Creates and manages Scaleway VPC Routes.
        For more information, see [the main documentation](https://www.scaleway.com/en/docs/vpc/concepts/).

        ## Example Usage

        ### With Instance

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        vpc01 = scaleway.network.Vpc("vpc01", name="tf-vpc-vpn")
        pn01 = scaleway.network.PrivateNetwork("pn01",
            name="tf-pn-vpn",
            ipv4_subnet={
                "subnet": "172.16.64.0/22",
            },
            vpc_id=vpc01.id)
        server01 = scaleway.instance.Server("server01",
            name="tf-server-vpn",
            type="PLAY2-MICRO",
            image="openvpn")
        pnic01 = scaleway.instance.PrivateNic("pnic01",
            private_network_id=pn01.id,
            server_id=server01.id)
        rt01 = scaleway.network.Route("rt01",
            vpc_id=vpc01.id,
            description="tf-route-vpn",
            tags=[
                "tf",
                "route",
            ],
            destination="10.0.0.0/24",
            nexthop_resource_id=pnic01.id)
        ```

        ### With Baremetal

        ```python
        import pulumi
        import pulumi_scaleway as scaleway
        import pulumiverse_scaleway as scaleway

        vpc01 = scaleway.network.Vpc("vpc01", name="tf-vpc-vpn")
        pn01 = scaleway.network.PrivateNetwork("pn01",
            name="tf-pn-vpn",
            ipv4_subnet={
                "subnet": "172.16.64.0/22",
            },
            vpc_id=vpc01.id)
        my_os = scaleway.elasticmetal.get_os(zone="fr-par-2",
            name="Ubuntu",
            version="22.04 LTS (Jammy Jellyfish)")
        my_offer = scaleway.elasticmetal.get_offer(zone="fr-par-2",
            name="EM-B112X-SSD")
        private_network = scaleway.elasticmetal.get_option(zone="fr-par-2",
            name="Private Network")
        my_key = scaleway.iam.get_ssh_key(name="main")
        my_server = scaleway.elasticmetal.Server("my_server",
            zone="fr-par-2",
            offer=my_offer.offer_id,
            os=my_os.os_id,
            ssh_key_ids=[my_key.id],
            options=[{
                "id": private_network.option_id,
            }],
            private_networks=[{
                "id": pn01.id,
            }])
        rt01 = scaleway.network.Route("rt01",
            vpc_id=vpc01.id,
            description="tf-route-vpn",
            tags=[
                "tf",
                "route",
            ],
            destination="10.0.0.0/24",
            nexthop_resource_id=my_server.private_networks[0].mapping_id)
        ```

        ## Import

        Routes can be imported using `{region}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:network/route:Route main fr-par/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] description: The route description.
        :param pulumi.Input[_builtins.str] destination: The destination of the route.
        :param pulumi.Input[_builtins.str] nexthop_private_network_id: The ID of the nexthop private network.
        :param pulumi.Input[_builtins.str] nexthop_resource_id: The ID of the nexthop resource.
        :param pulumi.Input[_builtins.str] region: `region`) The region of the route.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags to associate with the route.
        :param pulumi.Input[_builtins.str] vpc_id: The VPC ID the route belongs to.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: RouteArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Creates and manages Scaleway VPC Routes.
        For more information, see [the main documentation](https://www.scaleway.com/en/docs/vpc/concepts/).

        ## Example Usage

        ### With Instance

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        vpc01 = scaleway.network.Vpc("vpc01", name="tf-vpc-vpn")
        pn01 = scaleway.network.PrivateNetwork("pn01",
            name="tf-pn-vpn",
            ipv4_subnet={
                "subnet": "172.16.64.0/22",
            },
            vpc_id=vpc01.id)
        server01 = scaleway.instance.Server("server01",
            name="tf-server-vpn",
            type="PLAY2-MICRO",
            image="openvpn")
        pnic01 = scaleway.instance.PrivateNic("pnic01",
            private_network_id=pn01.id,
            server_id=server01.id)
        rt01 = scaleway.network.Route("rt01",
            vpc_id=vpc01.id,
            description="tf-route-vpn",
            tags=[
                "tf",
                "route",
            ],
            destination="10.0.0.0/24",
            nexthop_resource_id=pnic01.id)
        ```

        ### With Baremetal

        ```python
        import pulumi
        import pulumi_scaleway as scaleway
        import pulumiverse_scaleway as scaleway

        vpc01 = scaleway.network.Vpc("vpc01", name="tf-vpc-vpn")
        pn01 = scaleway.network.PrivateNetwork("pn01",
            name="tf-pn-vpn",
            ipv4_subnet={
                "subnet": "172.16.64.0/22",
            },
            vpc_id=vpc01.id)
        my_os = scaleway.elasticmetal.get_os(zone="fr-par-2",
            name="Ubuntu",
            version="22.04 LTS (Jammy Jellyfish)")
        my_offer = scaleway.elasticmetal.get_offer(zone="fr-par-2",
            name="EM-B112X-SSD")
        private_network = scaleway.elasticmetal.get_option(zone="fr-par-2",
            name="Private Network")
        my_key = scaleway.iam.get_ssh_key(name="main")
        my_server = scaleway.elasticmetal.Server("my_server",
            zone="fr-par-2",
            offer=my_offer.offer_id,
            os=my_os.os_id,
            ssh_key_ids=[my_key.id],
            options=[{
                "id": private_network.option_id,
            }],
            private_networks=[{
                "id": pn01.id,
            }])
        rt01 = scaleway.network.Route("rt01",
            vpc_id=vpc01.id,
            description="tf-route-vpn",
            tags=[
                "tf",
                "route",
            ],
            destination="10.0.0.0/24",
            nexthop_resource_id=my_server.private_networks[0].mapping_id)
        ```

        ## Import

        Routes can be imported using `{region}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:network/route:Route main fr-par/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param RouteArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(RouteArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 destination: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_private_network_id: Optional[pulumi.Input[_builtins.str]] = None,
                 nexthop_resource_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 vpc_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = RouteArgs.__new__(RouteArgs)

            __props__.__dict__["description"] = description
            __props__.__dict__["destination"] = destination
            __props__.__dict__["nexthop_private_network_id"] = nexthop_private_network_id
            __props__.__dict__["nexthop_resource_id"] = nexthop_resource_id
            __props__.__dict__["region"] = region
            __props__.__dict__["tags"] = tags
            if vpc_id is None and not opts.urn:
                raise TypeError("Missing required property 'vpc_id'")
            __props__.__dict__["vpc_id"] = vpc_id
            __props__.__dict__["created_at"] = None
            __props__.__dict__["updated_at"] = None
        alias_opts = pulumi.ResourceOptions(aliases=[pulumi.Alias(type_="scaleway:index/vpcRoute:VpcRoute")])
        opts = pulumi.ResourceOptions.merge(opts, alias_opts)
        super(Route, __self__).__init__(
            'scaleway:network/route:Route',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            created_at: Optional[pulumi.Input[_builtins.str]] = None,
            description: Optional[pulumi.Input[_builtins.str]] = None,
            destination: Optional[pulumi.Input[_builtins.str]] = None,
            nexthop_private_network_id: Optional[pulumi.Input[_builtins.str]] = None,
            nexthop_resource_id: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            updated_at: Optional[pulumi.Input[_builtins.str]] = None,
            vpc_id: Optional[pulumi.Input[_builtins.str]] = None) -> 'Route':
        """
        Get an existing Route resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] created_at: The date and time of the creation of the route (RFC 3339 format).
        :param pulumi.Input[_builtins.str] description: The route description.
        :param pulumi.Input[_builtins.str] destination: The destination of the route.
        :param pulumi.Input[_builtins.str] nexthop_private_network_id: The ID of the nexthop private network.
        :param pulumi.Input[_builtins.str] nexthop_resource_id: The ID of the nexthop resource.
        :param pulumi.Input[_builtins.str] region: `region`) The region of the route.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags to associate with the route.
        :param pulumi.Input[_builtins.str] updated_at: The date and time of the creation of the route (RFC 3339 format).
        :param pulumi.Input[_builtins.str] vpc_id: The VPC ID the route belongs to.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _RouteState.__new__(_RouteState)

        __props__.__dict__["created_at"] = created_at
        __props__.__dict__["description"] = description
        __props__.__dict__["destination"] = destination
        __props__.__dict__["nexthop_private_network_id"] = nexthop_private_network_id
        __props__.__dict__["nexthop_resource_id"] = nexthop_resource_id
        __props__.__dict__["region"] = region
        __props__.__dict__["tags"] = tags
        __props__.__dict__["updated_at"] = updated_at
        __props__.__dict__["vpc_id"] = vpc_id
        return Route(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[_builtins.str]:
        """
        The date and time of the creation of the route (RFC 3339 format).
        """
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The route description.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter
    def destination(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The destination of the route.
        """
        return pulumi.get(self, "destination")

    @_builtins.property
    @pulumi.getter(name="nexthopPrivateNetworkId")
    def nexthop_private_network_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The ID of the nexthop private network.
        """
        return pulumi.get(self, "nexthop_private_network_id")

    @_builtins.property
    @pulumi.getter(name="nexthopResourceId")
    def nexthop_resource_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The ID of the nexthop resource.
        """
        return pulumi.get(self, "nexthop_resource_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        `region`) The region of the route.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        The tags to associate with the route.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> pulumi.Output[_builtins.str]:
        """
        The date and time of the creation of the route (RFC 3339 format).
        """
        return pulumi.get(self, "updated_at")

    @_builtins.property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> pulumi.Output[_builtins.str]:
        """
        The VPC ID the route belongs to.
        """
        return pulumi.get(self, "vpc_id")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'InstanceGroupCapacityArgs',
    'InstanceGroupCapacityArgsDict',
    'InstanceGroupLoadBalancerArgs',
    'InstanceGroupLoadBalancerArgsDict',
    'InstancePolicyMetricArgs',
    'InstancePolicyMetricArgsDict',
    'InstanceTemplateVolumeArgs',
    'InstanceTemplateVolumeArgsDict',
    'InstanceTemplateVolumeFromEmptyArgs',
    'InstanceTemplateVolumeFromEmptyArgsDict',
    'InstanceTemplateVolumeFromSnapshotArgs',
    'InstanceTemplateVolumeFromSnapshotArgsDict',
]

MYPY = False

if not MYPY:
    class InstanceGroupCapacityArgsDict(TypedDict):
        cooldown_delay: NotRequired[pulumi.Input[_builtins.int]]
        """
        Time (in seconds) after a scaling action during which requests to carry out a new scaling action will be denied.
        """
        max_replicas: NotRequired[pulumi.Input[_builtins.int]]
        """
        The maximum count of Instances for the Instance group.
        """
        min_replicas: NotRequired[pulumi.Input[_builtins.int]]
        """
        The minimum count of Instances for the Instance group.
        """
elif False:
    InstanceGroupCapacityArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class InstanceGroupCapacityArgs:
    def __init__(__self__, *,
                 cooldown_delay: Optional[pulumi.Input[_builtins.int]] = None,
                 max_replicas: Optional[pulumi.Input[_builtins.int]] = None,
                 min_replicas: Optional[pulumi.Input[_builtins.int]] = None):
        """
        :param pulumi.Input[_builtins.int] cooldown_delay: Time (in seconds) after a scaling action during which requests to carry out a new scaling action will be denied.
        :param pulumi.Input[_builtins.int] max_replicas: The maximum count of Instances for the Instance group.
        :param pulumi.Input[_builtins.int] min_replicas: The minimum count of Instances for the Instance group.
        """
        if cooldown_delay is not None:
            pulumi.set(__self__, "cooldown_delay", cooldown_delay)
        if max_replicas is not None:
            pulumi.set(__self__, "max_replicas", max_replicas)
        if min_replicas is not None:
            pulumi.set(__self__, "min_replicas", min_replicas)

    @_builtins.property
    @pulumi.getter(name="cooldownDelay")
    def cooldown_delay(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Time (in seconds) after a scaling action during which requests to carry out a new scaling action will be denied.
        """
        return pulumi.get(self, "cooldown_delay")

    @cooldown_delay.setter
    def cooldown_delay(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "cooldown_delay", value)

    @_builtins.property
    @pulumi.getter(name="maxReplicas")
    def max_replicas(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The maximum count of Instances for the Instance group.
        """
        return pulumi.get(self, "max_replicas")

    @max_replicas.setter
    def max_replicas(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "max_replicas", value)

    @_builtins.property
    @pulumi.getter(name="minReplicas")
    def min_replicas(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The minimum count of Instances for the Instance group.
        """
        return pulumi.get(self, "min_replicas")

    @min_replicas.setter
    def min_replicas(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "min_replicas", value)


if not MYPY:
    class InstanceGroupLoadBalancerArgsDict(TypedDict):
        backend_ids: NotRequired[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]
        """
        The Load Balancer backend IDs.
        """
        id: NotRequired[pulumi.Input[_builtins.str]]
        """
        The ID of the Load Balancer.
        """
        private_network_id: NotRequired[pulumi.Input[_builtins.str]]
        """
        The ID of the Private Network attached to the Load Balancer.
        """
elif False:
    InstanceGroupLoadBalancerArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class InstanceGroupLoadBalancerArgs:
    def __init__(__self__, *,
                 backend_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 id: Optional[pulumi.Input[_builtins.str]] = None,
                 private_network_id: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] backend_ids: The Load Balancer backend IDs.
        :param pulumi.Input[_builtins.str] id: The ID of the Load Balancer.
        :param pulumi.Input[_builtins.str] private_network_id: The ID of the Private Network attached to the Load Balancer.
        """
        if backend_ids is not None:
            pulumi.set(__self__, "backend_ids", backend_ids)
        if id is not None:
            pulumi.set(__self__, "id", id)
        if private_network_id is not None:
            pulumi.set(__self__, "private_network_id", private_network_id)

    @_builtins.property
    @pulumi.getter(name="backendIds")
    def backend_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The Load Balancer backend IDs.
        """
        return pulumi.get(self, "backend_ids")

    @backend_ids.setter
    def backend_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "backend_ids", value)

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the Load Balancer.
        """
        return pulumi.get(self, "id")

    @id.setter
    def id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "id", value)

    @_builtins.property
    @pulumi.getter(name="privateNetworkId")
    def private_network_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the Private Network attached to the Load Balancer.
        """
        return pulumi.get(self, "private_network_id")

    @private_network_id.setter
    def private_network_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "private_network_id", value)


if not MYPY:
    class InstancePolicyMetricArgsDict(TypedDict):
        aggregate: pulumi.Input[_builtins.str]
        """
        How the values sampled for the `metric` should be aggregated.
        """
        name: pulumi.Input[_builtins.str]
        """
        Name or description of the metric policy.
        """
        operator: pulumi.Input[_builtins.str]
        """
        Operator used when comparing the threshold value of the chosen `metric` to the actual sampled and aggregated value.
        """
        cockpit_metric_name: NotRequired[pulumi.Input[_builtins.str]]
        """
        The custom metric to use for this policy. This must be stored in Scaleway Cockpit. The metric forms the basis of the condition that will be checked to determine whether a scaling action should be triggered
        """
        managed_metric: NotRequired[pulumi.Input[_builtins.str]]
        """
        The managed metric to use for this policy. These are available by default in Cockpit without any configuration or `node_exporter`. The chosen metric forms the basis of the condition that will be checked to determine whether a scaling action should be triggered.
        """
        sampling_range_min: NotRequired[pulumi.Input[_builtins.int]]
        """
        The Interval of time, in minutes, during which metric is sampled.
        """
        threshold: NotRequired[pulumi.Input[_builtins.int]]
        """
        The threshold value to measure the aggregated sampled `metric` value against. Combined with the `operator` field, determines whether a scaling action should be triggered.
        """
elif False:
    InstancePolicyMetricArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class InstancePolicyMetricArgs:
    def __init__(__self__, *,
                 aggregate: pulumi.Input[_builtins.str],
                 name: pulumi.Input[_builtins.str],
                 operator: pulumi.Input[_builtins.str],
                 cockpit_metric_name: Optional[pulumi.Input[_builtins.str]] = None,
                 managed_metric: Optional[pulumi.Input[_builtins.str]] = None,
                 sampling_range_min: Optional[pulumi.Input[_builtins.int]] = None,
                 threshold: Optional[pulumi.Input[_builtins.int]] = None):
        """
        :param pulumi.Input[_builtins.str] aggregate: How the values sampled for the `metric` should be aggregated.
        :param pulumi.Input[_builtins.str] name: Name or description of the metric policy.
        :param pulumi.Input[_builtins.str] operator: Operator used when comparing the threshold value of the chosen `metric` to the actual sampled and aggregated value.
        :param pulumi.Input[_builtins.str] cockpit_metric_name: The custom metric to use for this policy. This must be stored in Scaleway Cockpit. The metric forms the basis of the condition that will be checked to determine whether a scaling action should be triggered
        :param pulumi.Input[_builtins.str] managed_metric: The managed metric to use for this policy. These are available by default in Cockpit without any configuration or `node_exporter`. The chosen metric forms the basis of the condition that will be checked to determine whether a scaling action should be triggered.
        :param pulumi.Input[_builtins.int] sampling_range_min: The Interval of time, in minutes, during which metric is sampled.
        :param pulumi.Input[_builtins.int] threshold: The threshold value to measure the aggregated sampled `metric` value against. Combined with the `operator` field, determines whether a scaling action should be triggered.
        """
        pulumi.set(__self__, "aggregate", aggregate)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "operator", operator)
        if cockpit_metric_name is not None:
            pulumi.set(__self__, "cockpit_metric_name", cockpit_metric_name)
        if managed_metric is not None:
            pulumi.set(__self__, "managed_metric", managed_metric)
        if sampling_range_min is not None:
            pulumi.set(__self__, "sampling_range_min", sampling_range_min)
        if threshold is not None:
            pulumi.set(__self__, "threshold", threshold)

    @_builtins.property
    @pulumi.getter
    def aggregate(self) -> pulumi.Input[_builtins.str]:
        """
        How the values sampled for the `metric` should be aggregated.
        """
        return pulumi.get(self, "aggregate")

    @aggregate.setter
    def aggregate(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "aggregate", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Input[_builtins.str]:
        """
        Name or description of the metric policy.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def operator(self) -> pulumi.Input[_builtins.str]:
        """
        Operator used when comparing the threshold value of the chosen `metric` to the actual sampled and aggregated value.
        """
        return pulumi.get(self, "operator")

    @operator.setter
    def operator(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "operator", value)

    @_builtins.property
    @pulumi.getter(name="cockpitMetricName")
    def cockpit_metric_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The custom metric to use for this policy. This must be stored in Scaleway Cockpit. The metric forms the basis of the condition that will be checked to determine whether a scaling action should be triggered
        """
        return pulumi.get(self, "cockpit_metric_name")

    @cockpit_metric_name.setter
    def cockpit_metric_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "cockpit_metric_name", value)

    @_builtins.property
    @pulumi.getter(name="managedMetric")
    def managed_metric(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The managed metric to use for this policy. These are available by default in Cockpit without any configuration or `node_exporter`. The chosen metric forms the basis of the condition that will be checked to determine whether a scaling action should be triggered.
        """
        return pulumi.get(self, "managed_metric")

    @managed_metric.setter
    def managed_metric(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "managed_metric", value)

    @_builtins.property
    @pulumi.getter(name="samplingRangeMin")
    def sampling_range_min(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The Interval of time, in minutes, during which metric is sampled.
        """
        return pulumi.get(self, "sampling_range_min")

    @sampling_range_min.setter
    def sampling_range_min(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "sampling_range_min", value)

    @_builtins.property
    @pulumi.getter
    def threshold(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The threshold value to measure the aggregated sampled `metric` value against. Combined with the `operator` field, determines whether a scaling action should be triggered.
        """
        return pulumi.get(self, "threshold")

    @threshold.setter
    def threshold(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "threshold", value)


if not MYPY:
    class InstanceTemplateVolumeArgsDict(TypedDict):
        name: pulumi.Input[_builtins.str]
        """
        The name of the volume.
        """
        volume_type: pulumi.Input[_builtins.str]
        """
        The type of the volume.
        """
        boot: NotRequired[pulumi.Input[_builtins.bool]]
        """
        Force the Instance to boot on this volume.
        """
        from_empty: NotRequired[pulumi.Input['InstanceTemplateVolumeFromEmptyArgsDict']]
        """
        Volume instance template from empty
        """
        from_snapshot: NotRequired[pulumi.Input['InstanceTemplateVolumeFromSnapshotArgsDict']]
        """
        Volume instance template from snapshot
        """
        perf_iops: NotRequired[pulumi.Input[_builtins.int]]
        """
        The maximum IO/s expected, according to the different options available in stock (`5000 | 15000`).
        """
        tags: NotRequired[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]
        """
        The list of tags assigned to the volume.
        """
elif False:
    InstanceTemplateVolumeArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class InstanceTemplateVolumeArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[_builtins.str],
                 volume_type: pulumi.Input[_builtins.str],
                 boot: Optional[pulumi.Input[_builtins.bool]] = None,
                 from_empty: Optional[pulumi.Input['InstanceTemplateVolumeFromEmptyArgs']] = None,
                 from_snapshot: Optional[pulumi.Input['InstanceTemplateVolumeFromSnapshotArgs']] = None,
                 perf_iops: Optional[pulumi.Input[_builtins.int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        :param pulumi.Input[_builtins.str] name: The name of the volume.
        :param pulumi.Input[_builtins.str] volume_type: The type of the volume.
        :param pulumi.Input[_builtins.bool] boot: Force the Instance to boot on this volume.
        :param pulumi.Input['InstanceTemplateVolumeFromEmptyArgs'] from_empty: Volume instance template from empty
        :param pulumi.Input['InstanceTemplateVolumeFromSnapshotArgs'] from_snapshot: Volume instance template from snapshot
        :param pulumi.Input[_builtins.int] perf_iops: The maximum IO/s expected, according to the different options available in stock (`5000 | 15000`).
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The list of tags assigned to the volume.
        """
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "volume_type", volume_type)
        if boot is not None:
            pulumi.set(__self__, "boot", boot)
        if from_empty is not None:
            pulumi.set(__self__, "from_empty", from_empty)
        if from_snapshot is not None:
            pulumi.set(__self__, "from_snapshot", from_snapshot)
        if perf_iops is not None:
            pulumi.set(__self__, "perf_iops", perf_iops)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Input[_builtins.str]:
        """
        The name of the volume.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="volumeType")
    def volume_type(self) -> pulumi.Input[_builtins.str]:
        """
        The type of the volume.
        """
        return pulumi.get(self, "volume_type")

    @volume_type.setter
    def volume_type(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "volume_type", value)

    @_builtins.property
    @pulumi.getter
    def boot(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Force the Instance to boot on this volume.
        """
        return pulumi.get(self, "boot")

    @boot.setter
    def boot(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "boot", value)

    @_builtins.property
    @pulumi.getter(name="fromEmpty")
    def from_empty(self) -> Optional[pulumi.Input['InstanceTemplateVolumeFromEmptyArgs']]:
        """
        Volume instance template from empty
        """
        return pulumi.get(self, "from_empty")

    @from_empty.setter
    def from_empty(self, value: Optional[pulumi.Input['InstanceTemplateVolumeFromEmptyArgs']]):
        pulumi.set(self, "from_empty", value)

    @_builtins.property
    @pulumi.getter(name="fromSnapshot")
    def from_snapshot(self) -> Optional[pulumi.Input['InstanceTemplateVolumeFromSnapshotArgs']]:
        """
        Volume instance template from snapshot
        """
        return pulumi.get(self, "from_snapshot")

    @from_snapshot.setter
    def from_snapshot(self, value: Optional[pulumi.Input['InstanceTemplateVolumeFromSnapshotArgs']]):
        pulumi.set(self, "from_snapshot", value)

    @_builtins.property
    @pulumi.getter(name="perfIops")
    def perf_iops(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The maximum IO/s expected, according to the different options available in stock (`5000 | 15000`).
        """
        return pulumi.get(self, "perf_iops")

    @perf_iops.setter
    def perf_iops(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "perf_iops", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The list of tags assigned to the volume.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)


if not MYPY:
    class InstanceTemplateVolumeFromEmptyArgsDict(TypedDict):
        size: pulumi.Input[_builtins.int]
        """
        Size in GB of the new empty volume
        """
elif False:
    InstanceTemplateVolumeFromEmptyArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class InstanceTemplateVolumeFromEmptyArgs:
    def __init__(__self__, *,
                 size: pulumi.Input[_builtins.int]):
        """
        :param pulumi.Input[_builtins.int] size: Size in GB of the new empty volume
        """
        pulumi.set(__self__, "size", size)

    @_builtins.property
    @pulumi.getter
    def size(self) -> pulumi.Input[_builtins.int]:
        """
        Size in GB of the new empty volume
        """
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: pulumi.Input[_builtins.int]):
        pulumi.set(self, "size", value)


if not MYPY:
    class InstanceTemplateVolumeFromSnapshotArgsDict(TypedDict):
        snapshot_id: pulumi.Input[_builtins.str]
        """
        ID of the snapshot to clone
        """
        size: NotRequired[pulumi.Input[_builtins.int]]
        """
        Override size (in GB) of the cloned volume
        """
elif False:
    InstanceTemplateVolumeFromSnapshotArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class InstanceTemplateVolumeFromSnapshotArgs:
    def __init__(__self__, *,
                 snapshot_id: pulumi.Input[_builtins.str],
                 size: Optional[pulumi.Input[_builtins.int]] = None):
        """
        :param pulumi.Input[_builtins.str] snapshot_id: ID of the snapshot to clone
        :param pulumi.Input[_builtins.int] size: Override size (in GB) of the cloned volume
        """
        pulumi.set(__self__, "snapshot_id", snapshot_id)
        if size is not None:
            pulumi.set(__self__, "size", size)

    @_builtins.property
    @pulumi.getter(name="snapshotId")
    def snapshot_id(self) -> pulumi.Input[_builtins.str]:
        """
        ID of the snapshot to clone
        """
        return pulumi.get(self, "snapshot_id")

    @snapshot_id.setter
    def snapshot_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "snapshot_id", value)

    @_builtins.property
    @pulumi.getter
    def size(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Override size (in GB) of the cloned volume
        """
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "size", value)


